@@ -27,5 +27,5 @@ async def _main(args):
     print(f"Reading to {msgdefs.summary()}")
     for msgdef in msgdefs:
         if msgdef.read:
-            msg = await ebus.read(msgdef, prio=args.prio, ttl=args.ttl)
+            msg = await ebus.read(msgdef, defaultprio=9, setprio=args.prio, ttl=args.ttl)
             print_msg(msg)
