@@ -22,10 +22,11 @@ class Ebus:
     This instance connects to an EBUSD instance and allows to read, write or monitor.
     """
 
-    def __init__(self, host, port, timeout=10, scanwaitinterval=3, msgdefs=None):
+    def __init__(self, host, port, timeout=10, scanwaitinterval=5, scanwaits=5, msgdefs=None):
         self.connection = Connection(host=host, port=port, autoconnect=True, timeout=timeout)
         self.scanwaitinterval = scanwaitinterval
-        self._msgdefs = msgdefs
+        self.scanwaits = scanwaits
+        self.msgdefcodes = []
         self.msgdecoder = MsgDecoder(msgdefs or MsgDefs())
         _LOGGER.info(f"{self}")
 
@@ -35,8 +36,8 @@ class Ebus:
             args=(self.connection.host, self.connection.port),
             kwargs=(
                 ("timeout", self.timeout, 10),
-                ("scanwaitinterval", self.scanwaitinterval, 3),
-                ("msgdefs", self._msgdefs, None),
+                ("scanwaitinterval", self.scanwaitinterval, 5),
+                ("scanwaits", self.scanwaits, 5),
             ),
         )
 
@@ -75,8 +76,7 @@ class Ebus:
         while True:
             cnt = sum([1 async for line in self._request(_CMD_FINDMSGDEFS)])
             cnts.append(cnt)
-            if len(cnts) < 4 or cnts[-4] != cnts[-3] or cnts[-3] != cnts[-2] or cnts[-2] != cnts[-1]:
-                yield cnt
+            if len(cnts) < 4 or all(cnt == cnts[0] for cnt in cnts[1:]):
                 await asyncio.sleep(self.scanwaitinterval)
             else:
                 break
@@ -85,32 +85,45 @@ class Ebus:
         """
         Load Message Definitions from EBUSD.
 
-        Keyword Args:
-            scanwait(bool): Wait for EBUSD scan to complete
+        Alias for `load_msgdefcodes` and `decode_msgdefcodes`.
         """
-        _LOGGER.info("load_msgdefs()")
-        self.msgdefs.clear()
-        msgdefs = []
+        await self.load_msgdefcodes()
+        self.decode_msgdefcodes()
+
+    async def load_msgdefcodes(self):
+        """Load EBUS Message Definition Codes."""
+        _LOGGER.info("load_msgdefcodes()")
+        self.msgdefcodes = msgdefcodes = []
         async for line in self._request(_CMD_FINDMSGDEFS):
+            line = line.strip()
             if line:
-                try:
-                    msgdef = decode_msgdef(line)
-                    if not msgdef.circuit.startswith("scan"):
-                        msgdefs.append(msgdef)
-                except ValueError as exc:
-                    _LOGGER.warning(f"Cannot decode message definition ({exc})")
+                if not line.startswith("scan"):
+                    msgdefcodes.append(line)
+
+    def decode_msgdefcodes(self):
+        """Decode `msgdefcodes` and use as `msgdefs`."""
+        _LOGGER.info("decode_msgdefcodes()")
+        # Decode
+        msgdefs = []
+        for msgdefcode in self.msgdefcodes:
+            try:
+                msgdefs.append(decode_msgdef(msgdefcode))
+            except ValueError as exc:
+                _LOGGER.warning(f"Cannot decode message definition '{msgdefcode}' ({exc})")
+        # Sort
+        self.msgdefs.clear()
         for msgdef in sorted(msgdefs, key=lambda msgdef: (msgdef.circuit, msgdef.name)):
             self.msgdefs.add(msgdef)
 
-    async def read(self, msgdef, prio=False, ttl=None):
+    async def read(self, msgdef, defaultprio=None, setprio=False, ttl=None):
         """
         Read Message.
 
         Raises:
             ValueError: on decoder error
         """
-        _LOGGER.info(f"read({msgdef!r}, prio={prio!r}, ttl={ttl!r})")
-        prio = msgdef.prio if prio else None
+        _LOGGER.info(f"read({msgdef!r}, defaultprio={defaultprio!r}, setprio={setprio!r}, ttl={ttl!r})")
+        prio = msgdef.prio or defaultprio if setprio else None
         try:
             lines = tuple([line async for line in self._request("read", msgdef.name, c=msgdef.circuit, p=prio, m=ttl)])
         except CommandError as exc:
@@ -118,7 +131,7 @@ class Ebus:
         else:
             return self.msgdecoder.decode_value(msgdef, lines[0])
 
-    async def write(self, msgdef, value, ttl=None):
+    async def write(self, msgdef, value, ttl=0):
         """Write Message."""
         _LOGGER.info(f"write({msgdef!r}, value={value!r}, ttl={ttl!r})")
         if not msgdef.write:
@@ -135,7 +148,7 @@ class Ebus:
                 encvalue = fielddef.type_.encode(value)
                 values[fielddef.idx] = encvalue
         else:
-            values = [fielddef.type_.encode(value) for fielddef in msgdef.fields]
+            values = [str(fielddef.type_.encode(value)) for fielddef in msgdef.fields]
         # Write
         async for _ in self._request("write", msgdef.name, ";".join(values), c=msgdef.circuit, check=True):
             pass
@@ -152,7 +165,7 @@ class Ebus:
             if msg:
                 yield msg
 
-    async def observe(self, msgdefs=None, prio=False, ttl=None):
+    async def observe(self, msgdefs=None, defaultprio=None, setprio=False, ttl=None):
         """
         Observe.
 
@@ -160,14 +173,14 @@ class Ebus:
         Use `find` to get the latest data, if me missed any updates in the
         meantime and start listening
         """
-        _LOGGER.info(f"observe(msgdefs={msgdefs!r}, prio={prio!r}, ttl={ttl!r})")
+        _LOGGER.info(f"observe(msgdefs={msgdefs!r}, defaultprio={defaultprio!r}, setprio={setprio!r}, ttl={ttl!r})")
         msgdefs = msgdefs or self.msgdefs
         data = collections.defaultdict(lambda: None)
 
         # read all
         for msgdef in msgdefs:
             if msgdef.read:
-                msg = await self.read(msgdef, prio=prio, ttl=ttl)
+                msg = await self.read(msgdef, defaultprio=defaultprio, setprio=setprio, ttl=ttl)
                 if msg.valid:
                     msg = filter_msg(msg, msgdefs)
                 if msg.valid:
@@ -204,7 +217,7 @@ class Ebus:
 
     async def is_online(self):
         """Return if we are online."""
-        state = await get_state()
+        state = await self.get_state()
         return state == "ok"
 
     async def cmd(self, cmd, infinite=False, check=False):
